@@ -5,7 +5,7 @@
  * service.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
- * Copyright 2011, 2012, 2013
+ * Copyright 2011, 2012, 2013, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -718,7 +718,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "654321", NULL, 0 },
+                { "full", NULL, "654321", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -758,7 +758,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "123456", NULL, 0 },
+                { "full", NULL, "123456", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -811,7 +811,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "123456", NULL, 0 },
+                { "full", NULL, "123456", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -872,7 +872,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "123456", NULL, 0 },
+                { "full", NULL, "123456", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -935,7 +935,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "123456", NULL, 0 },
+                { "full", NULL, "123456", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -985,7 +985,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "full", NULL, "123456", NULL, 0 },
+                { "full", NULL, "123456", NULL, "DEVICEID", 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -1045,7 +1045,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "<krb5-principal>", 0, 0 },
             {
-                { "<userprinc>", "<password>", NULL, NULL, 0 },
+                { "<userprinc>", "<password>", NULL, NULL, NULL, 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -1183,7 +1183,7 @@ static const struct wat_login_test tests_default[] = {
         {
             { "krb5:webauth/example.com@EXAMPLE.COM", 0, 0 },
             {
-                { "<userprinc>", "<password>", NULL, NULL, 0 },
+                { "<userprinc>", "<password>", NULL, NULL, NULL, 0 },
                 EMPTY_TOKEN_LOGIN,
                 EMPTY_TOKEN_LOGIN
             },
@@ -1746,6 +1746,24 @@ main(void)
     for (i = 0; i < ARRAY_SIZE(tests_ignore_failure); i++)
         run_login_test(ctx, &tests_ignore_failure[i], ring, krbconf);
 
+    /*
+     * Re-run the basic tests with JSON.  Don't bother with the timeout and
+     * ignore error tests, since that functionality with JSON is tested by the
+     * userinfo test suite directly.
+     */
+#ifdef HAVE_JANSSON
+    user_config.timeout = 0;
+    user_config.ignore_failure = false;
+    user_config.command = "test-json";
+    user_config.json = true;
+    s = webauth_user_config(ctx, &user_config);
+    is_int(WA_ERR_NONE, s, "Setting user information protocol to JSON");
+    for (i = 0; i < ARRAY_SIZE(tests_default); i++)
+        run_login_test(ctx, &tests_default[i], ring, krbconf);
+#else
+    skip("not built with JSON support");
+#endif
+
     /* Clean up. */
     apr_terminate();
     return 0;
